"""
Wraps exceptions for authorizing and/or exchanging tokens.
"""


class CloudAuthzBaseException(ValueError):
    """
    Base class for all the exceptions thrown (or relayed) in CloudAuthnz.
    """
    def __init__(self, message, code=404, details=None):
        """
        Base class for all the exceptions thrown (or relayed) in CloudAuthnz.

        :type  message: string
        :param message: A description of the occurred error; it is either
                        generated by CloudAuthz, or extracted from the
                        cloud-based resource provider's response.

        :type  code:    int
        :param code:    HTTP response status code. If cloudauthz receives
                        an HTTP error code from the provider, it sets the
                        value of `code` to the received code; otherwise,
                        it assigns an HTTP code closest possible to the
                        cloud provider's error message.

        :type  details: object (vendor-dependent)
        :param details: Contains the details of the occurred error as
                        returned from the cloud-based resource provider.
                        Since the value of this property is set to
                        provider-returned details---without any
                        modification---its type can differ across providers.

        """
        self.message = message
        self.code = code
        self.details = details

    def __str__(self):
        return self.message


class InvalidTokenException(CloudAuthzBaseException):
    """
    Represents errors that occur during ID token utilization.
    """
    def __init__(self, message, code=404, details=None):
        CloudAuthzBaseException.__init__(self, message, code, details)


class ExpiredTokenException(CloudAuthzBaseException):
    """
    Represents errors that occur when utilizing expired tokens.
    """
    def __init__(self, message, code=404, details=None):
        CloudAuthzBaseException.__init__(self, message, code, details)


class AccessDeniedException(CloudAuthzBaseException):
    """
    Represents errors that occur when accessing resources that
    either the user is not allowed to, or if any of the
    resources is miss-spelled.
    """
    def __init__(self, message, code=403, details=None):
        CloudAuthzBaseException.__init__(self, message, code, details)

    def __str__(self):
        return self.message + ". You may not have access to the resource, or miss-spelled a configuration " \
                              "(e.g., incorrect amazon resource name, or an incorrect `aud` (Audience) is " \
                              "set on the resource provider)."


class InvalidRequestException(CloudAuthzBaseException):
    """
    Represents errors that occur when an invalid request is made
    to the resource provider. For instance, an invalid `Tenant name`
    on Microsoft Azure.
    """
    def __init__(self, message, code=404, details=None):
        CloudAuthzBaseException.__init__(self, message, code, details)
